# Upbit Open API
#
# ## REST API for Upbit Exchange - Base URL: [https://api.upbit.com] - Official Upbit API Documents: [https://docs.upbit.com] - Official Support email: [open-api@upbit.com] 
#
# OpenAPI spec version: 1.0.0
# Contact: ujhin942@gmail.com
# Generated by: https://github.com/swagger-api/swagger-codegen.git

#' @title Withdraw operations
#' @description swagger.Withdraw
#'
#' @field path Stores url path of the request.
#' @field apiClient Handles the client-server communication.
#' @field userAgent Set the user agent of the request.
#'
#' @importFrom R6 R6Class
#'
#' @section Methods:
#' \describe{
#'
#' withdraw_chance 출금 가능 정보
#'
#'
#' withdraw_coin 코인 출금하기
#'
#'
#' withdraw_info 개별 출금 조회
#'
#'
#' withdraw_info_all 출금 리스트 조회
#'
#'
#' withdraw_krw 원화 출금하기
#'
#' }
#'
#' @export
WithdrawApi <- R6::R6Class(
  'WithdrawApi',
  public = list(
    userAgent = "Swagger-Codegen/1.0.0/r",
    apiClient = NULL,
    initialize = function(apiClient){
      if (!missing(apiClient)) {
        self$apiClient <- apiClient
      }
      else {
        self$apiClient <- ApiClient$new()
      }
    },
    withdraw_chance = function(currency, ...){
      args <- list(...)
      queryParams <- list()
      headerParams <- character()

      if (!missing(`currency`)) {
        queryParams['currency'] <- currency
      }

      urlPath <- "/withdraws/chance"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body,
                                 ...)
      
      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        returnObject <- WithdrawChance$new()
        result <- returnObject$fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        Response$new("API client error", resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        Response$new("API server error", resp)
      }

    },
    withdraw_coin = function(currency, amount, address, secondary_address, transaction_type, ...){
      args <- list(...)
      queryParams <- list()
      headerParams <- character()

      body <- list(
          "currency" = currency,
          "amount" = amount,
          "address" = address,
          "secondary_address" = secondary_address,
          "transaction_type" = transaction_type
      )

      urlPath <- "/withdraws/coin"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "POST",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body,
                                 ...)
      
      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        returnObject <- WithdrawCoin$new()
        result <- returnObject$fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        Response$new("API client error", resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        Response$new("API server error", resp)
      }

    },
    withdraw_info = function(uuid, txid, currency, ...){
      args <- list(...)
      queryParams <- list()
      headerParams <- character()

      if (!missing(`uuid`)) {
        queryParams['uuid'] <- uuid
      }

      if (!missing(`txid`)) {
        queryParams['txid'] <- txid
      }

      if (!missing(`currency`)) {
        queryParams['currency'] <- currency
      }

      urlPath <- "/withdraw"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body,
                                 ...)
      
      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        returnObject <- Withdraw$new()
        result <- returnObject$fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        Response$new("API client error", resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        Response$new("API server error", resp)
      }

    },
    withdraw_info_all = function(currency, state, uuids, txids, limit, page, order_by, ...){
      args <- list(...)
      queryParams <- list()
      headerParams <- character()

      if (!missing(`currency`)) {
        queryParams['currency'] <- currency
      }

      if (!missing(`state`)) {
        queryParams['state'] <- state
      }

      if (!missing(`uuids`)) {
        queryParams['uuids'] <- uuids
      }

      if (!missing(`txids`)) {
        queryParams['txids'] <- txids
      }

      if (!missing(`limit`)) {
        queryParams['limit'] <- limit
      }

      if (!missing(`page`)) {
        queryParams['page'] <- page
      }

      if (!missing(`order_by`)) {
        queryParams['order_by'] <- order_by
      }

      urlPath <- "/withdraws"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "GET",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body,
                                 ...)
      
      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        returnObject <- Withdraw$new()
        result <- returnObject$fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        Response$new("API client error", resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        Response$new("API server error", resp)
      }

    },
    withdraw_krw = function(amount, ...){
      args <- list(...)
      queryParams <- list()
      headerParams <- character()

      body <- list(
          "amount" = amount
      )

      urlPath <- "/withdraws/krw"
      resp <- self$apiClient$callApi(url = paste0(self$apiClient$basePath, urlPath),
                                 method = "POST",
                                 queryParams = queryParams,
                                 headerParams = headerParams,
                                 body = body,
                                 ...)
      
      if (httr::status_code(resp) >= 200 && httr::status_code(resp) <= 299) {
        returnObject <- Withdraw$new()
        result <- returnObject$fromJSON(httr::content(resp, "text", encoding = "UTF-8"))
        Response$new(returnObject, resp)
      } else if (httr::status_code(resp) >= 400 && httr::status_code(resp) <= 499) {
        Response$new("API client error", resp)
      } else if (httr::status_code(resp) >= 500 && httr::status_code(resp) <= 599) {
        Response$new("API server error", resp)
      }

    }
  )
)
